import { Interface, BaseClass, Property, CodeGenerator } from 'pont-engine'

export default class MyGenerator extends CodeGenerator {
    getParams(inter: Interface) {
        const requestParams = inter.getRequestParams(this.surrounding)

        return `${requestParams.replace(
            'options?: any',
            `config?: axios.RequestConfig<Params${requestParams.includes('form') ? ' | FormData' : ''}>`,
        )}, options?: axios.RequestOptions`
    }

    getInterfaceContentInDeclaration(inter: Interface) {
        const requestParams = this.getParams(inter)
        const paramsCode = inter.getParamsCode('Params')

        return `
            export ${paramsCode}

            export type Response = ${inter.responseType.replace(/defs.Result\<(.*)\>/, '$1')};

            export const init: Response;

            export function request(${requestParams}): Promise<Response>;
        `
    }

    getBaseClassInDeclaration(base: BaseClass) {
        const originProps = base.properties

        base.properties = base.properties.map((prop) => {
            return new Property({
                ...prop,
                required: false,
            })
        })

        const result = super.getBaseClassInDeclaration(base)
        base.properties = originProps

        return result
    }

    getInterfaceContent(inter: Interface) {
        const method = inter.method.toUpperCase()
        const requestParams = this.getParams(inter)
        const paramsCode = inter.getParamsCode('Params', this.surrounding)

        return `
            /**
             * @desc ${inter.description}
             */

            import * as defs from '../../baseClass';
            import { defHttp } from '/@/utils/http/axios'

            export ${paramsCode}

            export const init = ${inter.response.getInitialValue()};

            export function request(${requestParams}) {
                return defHttp.request({
                    url: "${inter.path}",
                    method: '${method}',
                    ${
                        method === 'GET'
                            ? 'params'
                            : `data: ${requestParams.includes('form') ? 'form || params' : 'params'}`
                    },

                    ...config,
                }, options);
            }
        `
    }
}
